@@ -1,14 +1,26 @@
 """Nearest Neighbors graph functions"""
 
-# Author: Jake Vanderplas <vanderplas@astro.washington.edu>
-#         Tom Dupre la Tour
-#
-# License: BSD 3 clause (C) INRIA, University of Amsterdam
-from ._base import KNeighborsMixin, RadiusNeighborsMixin
-from ._base import NeighborsBase
-from ._unsupervised import NearestNeighbors
-from ..base import TransformerMixin, _ClassNamePrefixFeaturesOutMixin
-from ..utils.validation import check_is_fitted
+# Authors: The scikit-learn developers
+# SPDX-License-Identifier: BSD-3-Clause
+
+import itertools
+
+from sklearn.base import ClassNamePrefixFeaturesOutMixin, TransformerMixin, _fit_context
+from sklearn.neighbors._base import (
+    VALID_METRICS,
+    KNeighborsMixin,
+    NeighborsBase,
+    RadiusNeighborsMixin,
+)
+from sklearn.neighbors._unsupervised import NearestNeighbors
+from sklearn.utils._param_validation import (
+    Integral,
+    Interval,
+    Real,
+    StrOptions,
+    validate_params,
+)
+from sklearn.utils.validation import check_is_fitted
 
 
 def _check_params(X, metric, p, metric_params):
@@ -35,6 +47,19 @@ def _query_include_self(X, include_self, mode):
     return X
 
 
+@validate_params(
+    {
+        "X": ["array-like", "sparse matrix", KNeighborsMixin],
+        "n_neighbors": [Interval(Integral, 1, None, closed="left")],
+        "mode": [StrOptions({"connectivity", "distance"})],
+        "metric": [StrOptions(set(itertools.chain(*VALID_METRICS.values()))), callable],
+        "p": [Interval(Real, 0, None, closed="right"), None],
+        "metric_params": [dict, None],
+        "include_self": ["boolean", StrOptions({"auto"})],
+        "n_jobs": [Integral, None],
+    },
+    prefer_skip_nested_validation=False,  # metric is not validated yet
+)
 def kneighbors_graph(
     X,
     n_neighbors,
@@ -52,9 +77,8 @@ def kneighbors_graph(
 
     Parameters
     ----------
-    X : array-like of shape (n_samples, n_features) or BallTree
-        Sample data, in the form of a numpy array or a precomputed
-        :class:`BallTree`.
+    X : {array-like, sparse matrix} of shape (n_samples, n_features)
+        Sample data.
 
     n_neighbors : int
         Number of neighbors for each sample.
@@ -73,10 +97,11 @@ def kneighbors_graph(
         :class:`~sklearn.metrics.pairwise.distance_metrics` for valid metric
         values.
 
-    p : int, default=2
-        Power parameter for the Minkowski metric. When p = 1, this is
-        equivalent to using manhattan_distance (l1), and euclidean_distance
-        (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
+    p : float, default=2
+        Power parameter for the Minkowski metric. When p = 1, this is equivalent
+        to using manhattan_distance (l1), and euclidean_distance (l2) for p = 2.
+        For arbitrary p, minkowski_distance (l_p) is used. This parameter is expected
+        to be positive.
 
     metric_params : dict, default=None
         Additional keyword arguments for the metric function.
@@ -127,6 +152,19 @@ def kneighbors_graph(
     return X.kneighbors_graph(X=query, n_neighbors=n_neighbors, mode=mode)
 
 
+@validate_params(
+    {
+        "X": ["array-like", "sparse matrix", RadiusNeighborsMixin],
+        "radius": [Interval(Real, 0, None, closed="both")],
+        "mode": [StrOptions({"connectivity", "distance"})],
+        "metric": [StrOptions(set(itertools.chain(*VALID_METRICS.values()))), callable],
+        "p": [Interval(Real, 0, None, closed="right"), None],
+        "metric_params": [dict, None],
+        "include_self": ["boolean", StrOptions({"auto"})],
+        "n_jobs": [Integral, None],
+    },
+    prefer_skip_nested_validation=False,  # metric is not validated yet
+)
 def radius_neighbors_graph(
     X,
     radius,
@@ -147,9 +185,8 @@ def radius_neighbors_graph(
 
     Parameters
     ----------
-    X : array-like of shape (n_samples, n_features) or BallTree
-        Sample data, in the form of a numpy array or a precomputed
-        :class:`BallTree`.
+    X : {array-like, sparse matrix} of shape (n_samples, n_features)
+        Sample data.
 
     radius : float
         Radius of neighborhoods.
@@ -168,7 +205,7 @@ def radius_neighbors_graph(
         :class:`~sklearn.metrics.pairwise.distance_metrics` for valid metric
         values.
 
-    p : int, default=2
+    p : float, default=2
         Power parameter for the Minkowski metric. When p = 1, this is
         equivalent to using manhattan_distance (l1), and euclidean_distance
         (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
@@ -224,7 +261,7 @@ def radius_neighbors_graph(
 
 
 class KNeighborsTransformer(
-    _ClassNamePrefixFeaturesOutMixin, KNeighborsMixin, TransformerMixin, NeighborsBase
+    ClassNamePrefixFeaturesOutMixin, KNeighborsMixin, TransformerMixin, NeighborsBase
 ):
     """Transform X into a (weighted) graph of k nearest neighbors.
 
@@ -281,16 +318,17 @@ class KNeighborsTransformer(
 
         Distance matrices are not supported.
 
-    p : int, default=2
+    p : float, default=2
         Parameter for the Minkowski metric from
         sklearn.metrics.pairwise.pairwise_distances. When p = 1, this is
         equivalent to using manhattan_distance (l1), and euclidean_distance
         (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
+        This parameter is expected to be positive.
 
     metric_params : dict, default=None
         Additional keyword arguments for the metric function.
 
-    n_jobs : int, default=1
+    n_jobs : int, default=None
         The number of parallel jobs to run for neighbors search.
         If ``-1``, then the number of jobs is set to the number of CPU cores.
 
@@ -328,6 +366,12 @@ class KNeighborsTransformer(
     RadiusNeighborsTransformer : Transform X into a weighted graph of
         neighbors nearer than a radius.
 
+    Notes
+    -----
+    For an example of using :class:`~sklearn.neighbors.KNeighborsTransformer`
+    in combination with :class:`~sklearn.manifold.TSNE` see
+    :ref:`sphx_glr_auto_examples_neighbors_approximate_nearest_neighbors.py`.
+
     Examples
     --------
     >>> from sklearn.datasets import load_wine
@@ -341,6 +385,12 @@ class KNeighborsTransformer(
     (178, 178)
     """
 
+    _parameter_constraints: dict = {
+        **NeighborsBase._parameter_constraints,
+        "mode": [StrOptions({"distance", "connectivity"})],
+    }
+    _parameter_constraints.pop("radius")
+
     def __init__(
         self,
         *,
@@ -351,9 +401,9 @@ class KNeighborsTransformer(
         metric="minkowski",
         p=2,
         metric_params=None,
-        n_jobs=1,
+        n_jobs=None,
     ):
-        super(KNeighborsTransformer, self).__init__(
+        super().__init__(
             n_neighbors=n_neighbors,
             radius=None,
             algorithm=algorithm,
@@ -365,6 +415,10 @@ class KNeighborsTransformer(
         )
         self.mode = mode
 
+    @_fit_context(
+        # KNeighborsTransformer.metric is not validated yet
+        prefer_skip_nested_validation=False
+    )
     def fit(self, X, y=None):
         """Fit the k-nearest neighbors transformer from the training dataset.
 
@@ -431,16 +485,9 @@ class KNeighborsTransformer(
         """
         return self.fit(X).transform(X)
 
-    def _more_tags(self):
-        return {
-            "_xfail_checks": {
-                "check_methods_sample_order_invariance": "check is not applicable."
-            }
-        }
-
 
 class RadiusNeighborsTransformer(
-    _ClassNamePrefixFeaturesOutMixin,
+    ClassNamePrefixFeaturesOutMixin,
     RadiusNeighborsMixin,
     TransformerMixin,
     NeighborsBase,
@@ -498,16 +545,17 @@ class RadiusNeighborsTransformer(
 
         Distance matrices are not supported.
 
-    p : int, default=2
+    p : float, default=2
         Parameter for the Minkowski metric from
         sklearn.metrics.pairwise.pairwise_distances. When p = 1, this is
         equivalent to using manhattan_distance (l1), and euclidean_distance
         (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
+        This parameter is expected to be positive.
 
     metric_params : dict, default=None
         Additional keyword arguments for the metric function.
 
-    n_jobs : int, default=1
+    n_jobs : int, default=None
         The number of parallel jobs to run for neighbors search.
         If ``-1``, then the number of jobs is set to the number of CPU cores.
 
@@ -562,6 +610,12 @@ class RadiusNeighborsTransformer(
     [ 29  15 111  11  12]
     """
 
+    _parameter_constraints: dict = {
+        **NeighborsBase._parameter_constraints,
+        "mode": [StrOptions({"distance", "connectivity"})],
+    }
+    _parameter_constraints.pop("n_neighbors")
+
     def __init__(
         self,
         *,
@@ -572,9 +626,9 @@ class RadiusNeighborsTransformer(
         metric="minkowski",
         p=2,
         metric_params=None,
-        n_jobs=1,
+        n_jobs=None,
     ):
-        super(RadiusNeighborsTransformer, self).__init__(
+        super().__init__(
             n_neighbors=None,
             radius=radius,
             algorithm=algorithm,
@@ -586,6 +640,10 @@ class RadiusNeighborsTransformer(
         )
         self.mode = mode
 
+    @_fit_context(
+        # RadiusNeighborsTransformer.metric is not validated yet
+        prefer_skip_nested_validation=False
+    )
     def fit(self, X, y=None):
         """Fit the radius neighbors transformer from the training dataset.
 
@@ -649,10 +707,3 @@ class RadiusNeighborsTransformer(
             The matrix is of CSR format.
         """
         return self.fit(X).transform(X)
-
-    def _more_tags(self):
-        return {
-            "_xfail_checks": {
-                "check_methods_sample_order_invariance": "check is not applicable."
-            }
-        }
