@@ -15,23 +15,28 @@ from scipy import sparse
 from scipy.interpolate import BSpline
 from scipy.special import comb
 
-from ..base import BaseEstimator, TransformerMixin, _fit_context
-from ..utils import check_array
-from ..utils._param_validation import Interval, StrOptions
-from ..utils.fixes import parse_version, sp_version
-from ..utils.stats import _weighted_percentile
-from ..utils.validation import (
+from sklearn.base import BaseEstimator, TransformerMixin, _fit_context
+from sklearn.preprocessing._csr_polynomial_expansion import (
+    _calc_expanded_nnz,
+    _calc_total_nnz,
+    _csr_polynomial_expansion,
+)
+from sklearn.utils import check_array
+from sklearn.utils._array_api import (
+    _is_numpy_namespace,
+    get_namespace_and_device,
+    supported_float_dtypes,
+)
+from sklearn.utils._mask import _get_mask
+from sklearn.utils._param_validation import Interval, StrOptions
+from sklearn.utils.stats import _weighted_percentile
+from sklearn.utils.validation import (
     FLOAT_DTYPES,
     _check_feature_names_in,
     _check_sample_weight,
     check_is_fitted,
     validate_data,
 )
-from ._csr_polynomial_expansion import (
-    _calc_expanded_nnz,
-    _calc_total_nnz,
-    _csr_polynomial_expansion,
-)
 
 __all__ = [
     "PolynomialFeatures",
@@ -415,18 +420,18 @@ class PolynomialFeatures(TransformerMixin, BaseEstimator):
             `csr_matrix`.
         """
         check_is_fitted(self)
-
+        xp, _, device_ = get_namespace_and_device(X)
         X = validate_data(
             self,
             X,
             order="F",
-            dtype=FLOAT_DTYPES,
+            dtype=supported_float_dtypes(xp=xp, device=device_),
             reset=False,
             accept_sparse=("csr", "csc"),
         )
 
         n_samples, n_features = X.shape
-        max_int32 = np.iinfo(np.int32).max
+        max_int32 = xp.iinfo(xp.int32).max
         if sparse.issparse(X) and X.format == "csr":
             if self._max_degree > 3:
                 return self.transform(X.tocsc()).tocsr()
@@ -454,23 +459,6 @@ class PolynomialFeatures(TransformerMixin, BaseEstimator):
                 # edge case: deal with empty matrix
                 XP = sparse.csr_matrix((n_samples, 0), dtype=X.dtype)
             else:
-                # `scipy.sparse.hstack` breaks in scipy<1.9.2
-                # when `n_output_features_ > max_int32`
-                all_int32 = all(mat.indices.dtype == np.int32 for mat in to_stack)
-                if (
-                    sp_version < parse_version("1.9.2")
-                    and self.n_output_features_ > max_int32
-                    and all_int32
-                ):
-                    raise ValueError(  # pragma: no cover
-                        "In scipy versions `<1.9.2`, the function `scipy.sparse.hstack`"
-                        " produces negative columns when:\n1. The output shape contains"
-                        " `n_cols` too large to be represented by a 32bit signed"
-                        " integer.\n2. All sub-matrices to be stacked have indices of"
-                        " dtype `np.int32`.\nTo avoid this error, either use a version"
-                        " of scipy `>=1.9.2` or alter the `PolynomialFeatures`"
-                        " transformer to produce fewer than 2^31 output features"
-                    )
                 XP = sparse.hstack(to_stack, dtype=X.dtype, format="csr")
         elif sparse.issparse(X) and X.format == "csc" and self._max_degree < 4:
             return self.transform(X.tocsr()).tocsc()
@@ -496,8 +484,19 @@ class PolynomialFeatures(TransformerMixin, BaseEstimator):
         else:
             # Do as if _min_degree = 0 and cut down array after the
             # computation, i.e. use _n_out_full instead of n_output_features_.
-            XP = np.empty(
-                shape=(n_samples, self._n_out_full), dtype=X.dtype, order=self.order
+            order_kwargs = {}
+            if _is_numpy_namespace(xp=xp):
+                order_kwargs["order"] = self.order
+            elif self.order == "F":
+                raise ValueError(
+                    "PolynomialFeatures does not support order='F' for non-numpy arrays"
+                )
+
+            XP = xp.empty(
+                shape=(n_samples, self._n_out_full),
+                dtype=X.dtype,
+                device=device_,
+                **order_kwargs,
             )
 
             # What follows is a faster implementation of:
@@ -543,12 +542,18 @@ class PolynomialFeatures(TransformerMixin, BaseEstimator):
                         break
                     # XP[:, start:end] are terms of degree d - 1
                     # that exclude feature #feature_idx.
-                    np.multiply(
-                        XP[:, start:end],
-                        X[:, feature_idx : feature_idx + 1],
-                        out=XP[:, current_col:next_col],
-                        casting="no",
-                    )
+                    if _is_numpy_namespace(xp):
+                        # numpy performs this multiplication in place
+                        np.multiply(
+                            XP[:, start:end],
+                            X[:, feature_idx : feature_idx + 1],
+                            out=XP[:, current_col:next_col],
+                            casting="no",
+                        )
+                    else:
+                        XP[:, current_col:next_col] = xp.multiply(
+                            XP[:, start:end], X[:, feature_idx : feature_idx + 1]
+                        )
                     current_col = next_col
 
                 new_index.append(current_col)
@@ -557,19 +562,23 @@ class PolynomialFeatures(TransformerMixin, BaseEstimator):
             if self._min_degree > 1:
                 n_XP, n_Xout = self._n_out_full, self.n_output_features_
                 if self.include_bias:
-                    Xout = np.empty(
-                        shape=(n_samples, n_Xout), dtype=XP.dtype, order=self.order
+                    Xout = xp.empty(
+                        shape=(n_samples, n_Xout),
+                        dtype=XP.dtype,
+                        device=device_,
+                        **order_kwargs,
                     )
                     Xout[:, 0] = 1
                     Xout[:, 1:] = XP[:, n_XP - n_Xout + 1 :]
                 else:
-                    Xout = XP[:, n_XP - n_Xout :].copy()
+                    Xout = xp.asarray(XP[:, n_XP - n_Xout :], copy=True)
                 XP = Xout
         return XP
 
     def __sklearn_tags__(self):
         tags = super().__sklearn_tags__()
         tags.input_tags.sparse = True
+        tags.array_api_support = True
         return tags
 
 
@@ -638,6 +647,20 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
         Order of output array in the dense case. `'F'` order is faster to compute, but
         may slow down subsequent estimators.
 
+    handle_missing : {'error', 'zeros'}, default='error'
+        Specifies the way missing values are handled.
+
+        - 'error' : Raise an error if `np.nan` values are present during :meth:`fit`.
+        - 'zeros' : Encode splines of missing values with values `0`.
+
+        Note that `handle_missing='zeros'` differs from first imputing missing values
+        with zeros and then creating the spline basis. The latter creates spline basis
+        functions which have non-zero values at the missing values
+        whereas this option simply sets all spline basis function values to zero at the
+        missing values.
+
+        .. versionadded:: 1.8
+
     sparse_output : bool, default=False
         Will return sparse CSR matrix if set True else will return an array.
 
@@ -704,6 +727,7 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
         ],
         "include_bias": ["boolean"],
         "order": [StrOptions({"C", "F"})],
+        "handle_missing": [StrOptions({"error", "zeros"})],
         "sparse_output": ["boolean"],
     }
 
@@ -716,6 +740,7 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
         extrapolation="constant",
         include_bias=True,
         order="C",
+        handle_missing="error",
         sparse_output=False,
     ):
         self.n_knots = n_knots
@@ -724,11 +749,12 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
         self.extrapolation = extrapolation
         self.include_bias = include_bias
         self.order = order
+        self.handle_missing = handle_missing
         self.sparse_output = sparse_output
 
     @staticmethod
     def _get_base_knot_positions(X, n_knots=10, knots="uniform", sample_weight=None):
-        """Calculate base knot positions.
+        """Calculate base knot positions for `knots` either "uniform" or "quantile".
 
         Base knots such that first knot <= feature <= last knot. For the
         B-spline construction with scipy.interpolate.BSpline, 2*degree knots
@@ -745,14 +771,9 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
             )
 
             if sample_weight is None:
-                knots = np.percentile(X, percentile_ranks, axis=0)
+                knots = np.nanpercentile(X, percentile_ranks, axis=0)
             else:
-                knots = np.array(
-                    [
-                        _weighted_percentile(X, sample_weight, percentile_rank)
-                        for percentile_rank in percentile_ranks
-                    ]
-                )
+                knots = _weighted_percentile(X, sample_weight, percentile_ranks).T
 
         else:
             # knots == 'uniform':
@@ -760,8 +781,15 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
             # `else` is therefore safe.
             # Disregard observations with zero weight.
             mask = slice(None, None, 1) if sample_weight is None else sample_weight > 0
-            x_min = np.amin(X[mask], axis=0)
-            x_max = np.amax(X[mask], axis=0)
+            x_min = np.zeros(X.shape[1], dtype=np.float64)
+            x_max = np.zeros(X.shape[1], dtype=np.float64)
+            for feature_idx in range(X.shape[1]):
+                x = X[mask, feature_idx]
+                if np.all(np.isnan(x)):
+                    continue
+                else:
+                    x_min[feature_idx] = np.nanmin(x)
+                    x_max[feature_idx] = np.nanmax(x)
 
             knots = np.linspace(
                 start=x_min,
@@ -825,14 +853,26 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
         self : object
             Fitted transformer.
         """
-        X = validate_data(
-            self,
-            X,
-            reset=True,
-            accept_sparse=False,
-            ensure_min_samples=2,
-            ensure_2d=True,
-        )
+        try:
+            X = validate_data(
+                self,
+                X,
+                reset=True,
+                accept_sparse=False,
+                ensure_min_samples=2,
+                ensure_2d=True,
+                ensure_all_finite=(self.handle_missing != "zeros"),
+            )
+        except ValueError as e:
+            if "Input X contains NaN." in str(e) and self.handle_missing == "error":
+                raise ValueError(
+                    "Input X contains NaN values and `SplineTransformer` is configured "
+                    "to error in this case (handle_missing='error'). To avoid this "
+                    "error, set handle_missing='zeros' to encode missing values as "
+                    "splines with value 0 or ensure no missing values in X."
+                ) from e
+            raise e
+
         if sample_weight is not None:
             sample_weight = _check_sample_weight(sample_weight, X, dtype=X.dtype)
 
@@ -840,7 +880,10 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
 
         if isinstance(self.knots, str):
             base_knots = self._get_base_knot_positions(
-                X, n_knots=self.n_knots, knots=self.knots, sample_weight=sample_weight
+                X,
+                n_knots=self.n_knots,
+                knots=self.knots,
+                sample_weight=sample_weight,
             )
         else:
             base_knots = check_array(self.knots, dtype=np.float64)
@@ -948,25 +991,19 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
         """
         check_is_fitted(self)
 
-        X = validate_data(self, X, reset=False, accept_sparse=False, ensure_2d=True)
+        X = validate_data(
+            self,
+            X,
+            reset=False,
+            accept_sparse=False,
+            ensure_2d=True,
+            ensure_all_finite=(self.handle_missing != "zeros"),
+        )
 
         n_samples, n_features = X.shape
         n_splines = self.bsplines_[0].c.shape[1]
         degree = self.degree
 
-        # TODO: Remove this condition, once scipy 1.10 is the minimum version.
-        #       Only scipy => 1.10 supports design_matrix(.., extrapolate=..).
-        #       The default (implicit in scipy < 1.10) is extrapolate=False.
-        scipy_1_10 = sp_version >= parse_version("1.10.0")
-        # Note: self.bsplines_[0].extrapolate is True for extrapolation in
-        # ["periodic", "continue"]
-        if scipy_1_10:
-            use_sparse = self.sparse_output
-            kwargs_extrapolate = {"extrapolate": self.bsplines_[0].extrapolate}
-        else:
-            use_sparse = self.sparse_output and not self.bsplines_[0].extrapolate
-            kwargs_extrapolate = dict()
-
         # Note that scipy BSpline returns float64 arrays and converts input
         # x=X[:, i] to c-contiguous float64.
         n_out = self.n_features_out_ + n_features * (1 - self.include_bias)
@@ -974,13 +1011,15 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
             dtype = X.dtype
         else:
             dtype = np.float64
-        if use_sparse:
+        if self.sparse_output:
             output_list = []
         else:
             XBS = np.zeros((n_samples, n_out), dtype=dtype, order=self.order)
 
-        for i in range(n_features):
-            spl = self.bsplines_[i]
+        for feature_idx in range(n_features):
+            spl = self.bsplines_[feature_idx]
+            # Get indicator for nan values in the current column.
+            nan_row_indices = np.flatnonzero(_get_mask(X[:, feature_idx], np.nan))
 
             if self.extrapolation in ("continue", "error", "periodic"):
                 if self.extrapolation == "periodic":
@@ -989,17 +1028,47 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
                     # This is equivalent to BSpline(.., extrapolate="periodic")
                     # for scipy>=1.0.0.
                     n = spl.t.size - spl.k - 1
-                    # Assign to new array to avoid inplace operation
-                    x = spl.t[spl.k] + (X[:, i] - spl.t[spl.k]) % (
-                        spl.t[n] - spl.t[spl.k]
-                    )
-                else:
-                    x = X[:, i]
+                    if spl.t[n] - spl.t[spl.k] > 0:
+                        # Assign to new array to avoid inplace operation
+                        x = spl.t[spl.k] + (X[:, feature_idx] - spl.t[spl.k]) % (
+                            spl.t[n] - spl.t[spl.k]
+                        )
+                    else:
+                        # This can happen if the column has a single non-nan
+                        # value. Treat as a constant feature.
+                        x = np.zeros_like(X[:, feature_idx])
+                else:  # self.extrapolation in ("continue", "error")
+                    x = X[:, feature_idx]
 
-                if use_sparse:
+                if self.sparse_output:
+                    # We replace the nan values in the input column by some
+                    # arbitrary, in-range, numerical value since
+                    # BSpline.design_matrix() would otherwise raise on any nan
+                    # value in its input. The spline encoded values in
+                    # the output of that function that correspond to missing
+                    # values in the original input will be replaced by 0.0
+                    # afterwards.
+                    #
+                    # Note that in the following we use np.nanmin(x) as the
+                    # input replacement to make sure that this code works even
+                    # when `extrapolation == "error"`. Any other choice of
+                    # in-range value would have worked work since the
+                    # corresponding values in the array are replaced by zeros.
+                    if nan_row_indices.size == x.size:
+                        # The column is all np.nan valued. Replace it by a
+                        # constant column with an arbitrary non-nan value
+                        # inside so that it is encoded as constant column.
+                        x = np.zeros_like(x)  # avoid mutation of input data
+                    elif nan_row_indices.shape[0] > 0:
+                        x = x.copy()  # avoid mutation of input data
+                        x[nan_row_indices] = np.nanmin(x)
+
+                    # Note: self.bsplines_[0].extrapolate is True for extrapolation in
+                    # ["periodic", "continue"]
                     XBS_sparse = BSpline.design_matrix(
-                        x, spl.t, spl.k, **kwargs_extrapolate
+                        x, spl.t, spl.k, self.bsplines_[0].extrapolate
                     )
+
                     if self.extrapolation == "periodic":
                         # See the construction of coef in fit. We need to add the last
                         # degree spline basis function to the first degree ones and
@@ -1008,72 +1077,113 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
                         XBS_sparse = XBS_sparse.tolil()
                         XBS_sparse[:, :degree] += XBS_sparse[:, -degree:]
                         XBS_sparse = XBS_sparse[:, :-degree]
+
+                    if nan_row_indices.shape[0] > 0:
+                        # Note: See comment about SparseEfficiencyWarning below.
+                        XBS = XBS_sparse.tolil()
+
                 else:
-                    XBS[:, (i * n_splines) : ((i + 1) * n_splines)] = spl(x)
+                    XBS[
+                        :, (feature_idx * n_splines) : ((feature_idx + 1) * n_splines)
+                    ] = spl(x)
+
+                # Replace any indicated values with 0:
+                if nan_row_indices.shape[0] > 0:
+                    for spline_idx in range(n_splines):
+                        output_feature_idx = n_splines * feature_idx + spline_idx
+                        XBS[
+                            nan_row_indices, output_feature_idx : output_feature_idx + 1
+                        ] = 0
+                    if self.sparse_output:
+                        XBS_sparse = XBS
+
             else:  # extrapolation in ("constant", "linear")
                 xmin, xmax = spl.t[degree], spl.t[-degree - 1]
                 # spline values at boundaries
                 f_min, f_max = spl(xmin), spl(xmax)
-                mask = (xmin <= X[:, i]) & (X[:, i] <= xmax)
-                if use_sparse:
-                    mask_inv = ~mask
-                    x = X[:, i].copy()
-                    # Set some arbitrary values outside boundary that will be reassigned
-                    # later.
-                    x[mask_inv] = spl.t[self.degree]
+                # Values outside of the feature range during fit and nan values get
+                # filtered out:
+                inside_range_mask = (xmin <= X[:, feature_idx]) & (
+                    X[:, feature_idx] <= xmax
+                )
+
+                if self.sparse_output:
+                    outside_range_mask = ~inside_range_mask
+                    x = X[:, feature_idx].copy()
+                    # Set to some arbitrary value within the range of values
+                    # observed on the training set before calling
+                    # BSpline.design_matrix. Those transformed will be
+                    # reassigned later when handling with extrapolation.
+                    x[outside_range_mask] = xmin
                     XBS_sparse = BSpline.design_matrix(x, spl.t, spl.k)
                     # Note: Without converting to lil_matrix we would get:
                     # scipy.sparse._base.SparseEfficiencyWarning: Changing the sparsity
                     # structure of a csr_matrix is expensive. lil_matrix is more
                     # efficient.
-                    if np.any(mask_inv):
+                    if np.any(outside_range_mask):
                         XBS_sparse = XBS_sparse.tolil()
-                        XBS_sparse[mask_inv, :] = 0
+                        XBS_sparse[outside_range_mask, :] = 0
+
                 else:
-                    XBS[mask, (i * n_splines) : ((i + 1) * n_splines)] = spl(X[mask, i])
+                    XBS[
+                        inside_range_mask,
+                        (feature_idx * n_splines) : ((feature_idx + 1) * n_splines),
+                    ] = spl(X[inside_range_mask, feature_idx])
 
             # Note for extrapolation:
             # 'continue' is already returned as is by scipy BSplines
             if self.extrapolation == "error":
-                # BSpline with extrapolate=False does not raise an error, but
-                # outputs np.nan.
-                if (use_sparse and np.any(np.isnan(XBS_sparse.data))) or (
-                    not use_sparse
-                    and np.any(
-                        np.isnan(XBS[:, (i * n_splines) : ((i + 1) * n_splines)])
+                has_nan_output_values = False
+                if self.sparse_output:
+                    # Early convert to CSR as the sparsity structure of this
+                    # block should not change anymore. This is needed to be able
+                    # to safely assume that `.data` is a 1D array.
+                    XBS_sparse = XBS_sparse.tocsr()
+                    has_nan_output_values = np.any(np.isnan(XBS_sparse.data))
+                else:
+                    output_features = slice(
+                        feature_idx * n_splines, (feature_idx + 1) * n_splines
                     )
-                ):
+                    has_nan_output_values = np.any(np.isnan(XBS[:, output_features]))
+
+                if has_nan_output_values:
                     raise ValueError(
-                        "X contains values beyond the limits of the knots."
+                        "`X` contains values beyond the limits of the knots."
                     )
+
             elif self.extrapolation == "constant":
                 # Set all values beyond xmin and xmax to the value of the
                 # spline basis functions at those two positions.
                 # Only the first degree and last degree number of splines
                 # have non-zero values at the boundaries.
 
-                mask = X[:, i] < xmin
-                if np.any(mask):
-                    if use_sparse:
+                below_xmin_mask = X[:, feature_idx] < xmin
+                if np.any(below_xmin_mask):
+                    if self.sparse_output:
                         # Note: See comment about SparseEfficiencyWarning above.
                         XBS_sparse = XBS_sparse.tolil()
-                        XBS_sparse[mask, :degree] = f_min[:degree]
+                        XBS_sparse[below_xmin_mask, :degree] = f_min[:degree]
 
-                    else:
-                        XBS[mask, (i * n_splines) : (i * n_splines + degree)] = f_min[
-                            :degree
-                        ]
-
-                mask = X[:, i] > xmax
-                if np.any(mask):
-                    if use_sparse:
-                        # Note: See comment about SparseEfficiencyWarning above.
-                        XBS_sparse = XBS_sparse.tolil()
-                        XBS_sparse[mask, -degree:] = f_max[-degree:]
                     else:
                         XBS[
-                            mask,
-                            ((i + 1) * n_splines - degree) : ((i + 1) * n_splines),
+                            below_xmin_mask,
+                            (feature_idx * n_splines) : (
+                                feature_idx * n_splines + degree
+                            ),
+                        ] = f_min[:degree]
+
+                above_xmax_mask = X[:, feature_idx] > xmax
+                if np.any(above_xmax_mask):
+                    if self.sparse_output:
+                        # Note: See comment about SparseEfficiencyWarning above.
+                        XBS_sparse = XBS_sparse.tolil()
+                        XBS_sparse[above_xmax_mask, -degree:] = f_max[-degree:]
+                    else:
+                        XBS[
+                            above_xmax_mask,
+                            ((feature_idx + 1) * n_splines - degree) : (
+                                (feature_idx + 1) * n_splines
+                            ),
                         ] = f_max[-degree:]
 
             elif self.extrapolation == "linear":
@@ -1091,58 +1201,45 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
                     # boundary. For degree=0 it is the same as 'constant'.
                     degree += 1
                 for j in range(degree):
-                    mask = X[:, i] < xmin
-                    if np.any(mask):
-                        linear_extr = f_min[j] + (X[mask, i] - xmin) * fp_min[j]
-                        if use_sparse:
+                    below_xmin_mask = X[:, feature_idx] < xmin
+                    if np.any(below_xmin_mask):
+                        linear_extr = (
+                            f_min[j]
+                            + (X[below_xmin_mask, feature_idx] - xmin) * fp_min[j]
+                        )
+                        if self.sparse_output:
                             # Note: See comment about SparseEfficiencyWarning above.
                             XBS_sparse = XBS_sparse.tolil()
-                            XBS_sparse[mask, j] = linear_extr
+                            XBS_sparse[below_xmin_mask, j] = linear_extr
                         else:
-                            XBS[mask, i * n_splines + j] = linear_extr
+                            XBS[below_xmin_mask, feature_idx * n_splines + j] = (
+                                linear_extr
+                            )
 
-                    mask = X[:, i] > xmax
-                    if np.any(mask):
+                    above_xmax_mask = X[:, feature_idx] > xmax
+                    if np.any(above_xmax_mask):
                         k = n_splines - 1 - j
-                        linear_extr = f_max[k] + (X[mask, i] - xmax) * fp_max[k]
-                        if use_sparse:
+                        linear_extr = (
+                            f_max[k]
+                            + (X[above_xmax_mask, feature_idx] - xmax) * fp_max[k]
+                        )
+                        if self.sparse_output:
                             # Note: See comment about SparseEfficiencyWarning above.
                             XBS_sparse = XBS_sparse.tolil()
-                            XBS_sparse[mask, k : k + 1] = linear_extr[:, None]
+                            XBS_sparse[above_xmax_mask, k : k + 1] = linear_extr[
+                                :, None
+                            ]
                         else:
-                            XBS[mask, i * n_splines + k] = linear_extr
+                            XBS[above_xmax_mask, feature_idx * n_splines + k] = (
+                                linear_extr
+                            )
 
-            if use_sparse:
+            if self.sparse_output:
                 XBS_sparse = XBS_sparse.tocsr()
                 output_list.append(XBS_sparse)
 
-        if use_sparse:
-            # TODO: Remove this conditional error when the minimum supported version of
-            # SciPy is 1.9.2
-            # `scipy.sparse.hstack` breaks in scipy<1.9.2
-            # when `n_features_out_ > max_int32`
-            max_int32 = np.iinfo(np.int32).max
-            all_int32 = True
-            for mat in output_list:
-                all_int32 &= mat.indices.dtype == np.int32
-            if (
-                sp_version < parse_version("1.9.2")
-                and self.n_features_out_ > max_int32
-                and all_int32
-            ):
-                raise ValueError(
-                    "In scipy versions `<1.9.2`, the function `scipy.sparse.hstack`"
-                    " produces negative columns when:\n1. The output shape contains"
-                    " `n_cols` too large to be represented by a 32bit signed"
-                    " integer.\n. All sub-matrices to be stacked have indices of"
-                    " dtype `np.int32`.\nTo avoid this error, either use a version"
-                    " of scipy `>=1.9.2` or alter the `SplineTransformer`"
-                    " transformer to produce fewer than 2^31 output features"
-                )
+        if self.sparse_output:
             XBS = sparse.hstack(output_list, format="csr")
-        elif self.sparse_output:
-            # TODO: Remove ones scipy 1.10 is the minimum version. See comments above.
-            XBS = sparse.csr_matrix(XBS)
 
         if self.include_bias:
             return XBS
@@ -1151,3 +1248,8 @@ class SplineTransformer(TransformerMixin, BaseEstimator):
             # We chose the last one.
             indices = [j for j in range(XBS.shape[1]) if (j + 1) % n_splines != 0]
             return XBS[:, indices]
+
+    def __sklearn_tags__(self):
+        tags = super().__sklearn_tags__()
+        tags.input_tags.allow_nan = self.handle_missing == "zeros"
+        return tags
